package com.jdcloud.market.sdk.core.util;

import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;

/**
 * @DESC :
 * @Author : WangLei
 * @Date : 2019/3/22.
 */
public class CommonUtil {

    /**
     * 获取token
     *
     * @param request
     * @param key
     * @return
     * @throws UnsupportedEncodingException
     */
    public static String getToken(HttpServletRequest request, String key) throws UnsupportedEncodingException {
        TreeMap<String, String> map = getReqParamMap(request);
        StringBuilder stringBuilder = getSignString(map, null);
        stringBuilder.append("&key=");
        stringBuilder.append(key);
        return Md5Util.md5Encode(stringBuilder.toString(), "UTF-8");
    }


    /**
     * 获取paramMap
     *
     * @param request
     * @return
     */
    public static TreeMap<String, String> getReqParamMap(HttpServletRequest request) {
        TreeMap<String, String> map = new TreeMap<>();
        Enumeration paramNames = request.getParameterNames();
        while (paramNames.hasMoreElements()) {
            String paramName = (String) paramNames.nextElement();
            String[] paramValues = request.getParameterValues(paramName);
            if (paramValues.length == 1) {
                String paramValue = paramValues[0];
                map.put(paramName, paramValue);
            }
        }

        return map;
    }

    /**
     * 获取签名字符串
     *
     * @param params
     * @param charset
     * @return
     * @throws UnsupportedEncodingException
     */
    private static StringBuilder getSignString(TreeMap<String, String> params, String charset) throws UnsupportedEncodingException {
        if (params == null || params.isEmpty()) {
            throw new IllegalArgumentException("sign params must be not empty!");
        }

        StringBuilder query = new StringBuilder();
        Set<Map.Entry<String, String>> entries = params.entrySet();
        boolean hasParam = false;
        for (Map.Entry<String, String> entry : entries) {
            String name = entry.getKey();
            String value = entry.getValue() == null ? "" : entry.getValue();
            // 忽略参数名为空的参数，参数值为空需要传递EmptyString
            if ("token".equalsIgnoreCase(name)) {
                continue;
            }
            if (!StringUtils.isEmpty(name)) {
                if (hasParam) {
                    query.append("&");
                } else {
                    hasParam = true;
                }
                query.append(name).append("=");
                if (StringUtils.isEmpty(charset)) {
                    query.append(value);
                } else {
                    query.append(URLEncoder.encode(value, charset));
                }
            }
        }

        return query;
    }

}
