package com.jdcloud.market.sdk.samples.controller;

import com.alibaba.fastjson.JSON;
import com.jdcloud.market.sdk.core.service.IJdcloudMarketSaasService;
import com.jdcloud.market.sdk.core.util.CommonUtil;
import com.jdcloud.market.sdk.core.vo.CreateInstanceReq;
import com.jdcloud.market.sdk.core.vo.UnifiedResp;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.util.Objects;
import java.util.TreeMap;

/**
 * Created with IntelliJ IDEA.
 * Date: 2019/3/22
 * Time: 11:34
 * 云市场主动通知（模拟服务商）此代码仅供参考，可根据自身实际情况开发
 * 参考：https://docs.jdcloud.com/cn/marketplace/notify-interface-standard
 */
@RestController
@RequestMapping("/saas/")
public class MockSaasNotificationController {
    private static Logger logger = LoggerFactory.getLogger(MockSaasNotificationController.class);

    @Value("${jdcloud.market.key}")
    private String key;

    @Autowired
    private IJdcloudMarketSaasService jdcloudMarketSaasService;

    @GetMapping(value = "/mock")
    public Object saasMock(HttpServletRequest request) throws UnsupportedEncodingException {

        TreeMap<String, String> paramMap = CommonUtil.getReqParamMap(request);
        String token = paramMap.get("token");
        if (Objects.isNull(token) || !token.equals(CommonUtil.getToken(request, key))) {
            logger.error("token参数有误");
            return UnifiedResp.fail("token参数有误");
        }

        CreateInstanceReq saasInstanceReq = JSON.parseObject(JSON.toJSONString(paramMap), CreateInstanceReq.class);
        if (saasInstanceReq == null) {
            logger.error("请求参数有误");
            return UnifiedResp.fail("请求参数有误");
        }

        if (StringUtils.equalsIgnoreCase(saasInstanceReq.getAction(), "createInstance")) {
            return jdcloudMarketSaasService.createInstance(saasInstanceReq);
        } else if (StringUtils.equalsIgnoreCase(saasInstanceReq.getAction(), "renewInstance")) {
            return jdcloudMarketSaasService.renewInstance(saasInstanceReq);
        } else if (StringUtils.equalsIgnoreCase(saasInstanceReq.getAction(), "releaseInstance")) {
            return jdcloudMarketSaasService.releaseInstance(saasInstanceReq);
        } else if (StringUtils.equalsIgnoreCase(saasInstanceReq.getAction(), "upgradeInstance")) {
            return jdcloudMarketSaasService.upgradeInstance(saasInstanceReq);
        } else if (StringUtils.equalsIgnoreCase(saasInstanceReq.getAction(), "dilateInstance")) {
            return jdcloudMarketSaasService.dilateInstance(saasInstanceReq);
        } else if (StringUtils.equalsIgnoreCase(saasInstanceReq.getAction(), "verify")) {
            return jdcloudMarketSaasService.login(saasInstanceReq);
        } else if (StringUtils.equalsIgnoreCase(saasInstanceReq.getAction(), "expiredInstance")) {
            return jdcloudMarketSaasService.expiredInstance(saasInstanceReq);
        } else {
            logger.error("不支持此方法");
            return UnifiedResp.fail("不支持此方法");
        }

    }

}
